# Feature toggle

This module gives non developers ability to show or hide features created and deployed
by developers.

For a full description of the module, visit the
[project page](https://www.drupal.org/project/feature_toggle).

Submit bug reports and feature suggestions, or track changes in the
[issue queue](https://www.drupal.org/project/issues/feature_toggle).


## Table of contents

- Requirements
- Installation
- Configuration
- Maintainers


## Requirements

This module requires no modules outside of Drupal core.


## Installation

Install as you would normally install a contributed Drupal module. For further
information, see
[Installing Drupal Modules](https://www.drupal.org/docs/extending-drupal/installing-drupal-modules).


## Configuration

Go to Administration » Configuration » System » Feature Toggle and you will see a listing page
which will allow you to enable or disable a feature.

### Manage Features
From the listing page above, Site Administrators are capable to define new features or
Edit/Delete the existing ones.

### How to get the status of a feature in PHP

```php
$status = \Drupal::service('feature_toggle.feature_status')->getStatus('some_feature');
```

### How to use an EventSubscriber to subscribe to FeatureUpdateEvents::UPDATE events

Add the event subscriber service definition to _example_module.services.yml_:
```yaml
services:
  example_module.event_subscriber:
    class: \Drupal\example_module\EventSubscriber\ExampleModuleEventSubscriber
    tags:
      - { name: event_subscriber }
```
Create your event subscriber class:

```php
namespace Drupal\feature_toggle_example\EventSubscriber;

use Drupal\feature_toggle\Event\FeatureUpdateEvent;
use Drupal\feature_toggle\Event\FeatureUpdateEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class ExampleModuleEventSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events = [];
    $events[FeatureUpdateEvents::UPDATE][] = 'featureUpdate';
    return $events;
  }

  /**
   * Event callback to react to FeatureUpdateEvents::UPDATE event.
   */
  public function featureUpdate(FeatureUpdateEvent $event) {
    \Drupal::logger('feature_toggle_example')->debug($event->feature()->name());
    \Drupal::logger('feature_toggle_example')->debug($event->feature()->label());
    \Drupal::logger('feature_toggle_example')->debug($event->status());
  }

}

```

### Define access to Custom routes based on feature status
It is possible to define route access rules based on the feature status.

In your my_module.routing.yml file:
```yaml
my_module.feature_route_enabled:
  path: '/access/if/feature/enabled'
  defaults:
    _form: '\Drupal\my_module\Form\MyModuleForm'
    _title: 'Feature Enabled'
  requirements:
    _feature_toggle: my_feature.1

my_module.feature_route_disabled:
  path: '/access/if/feature/disabled'
  defaults:
    _form: '\Drupal\my_module\Form\MyModuleForm'
    _title: 'Feature Disabled'
  requirements:
    _feature_toggle: my_feature.0
```
The 1st route will be accessible if `my_feature` is enabled, while the second one
is accessible if the feature is disabled.

**Note:** The `.1` is optional, given that by default the module assumes that access is
given to enabled features.

### Define Access to Views based on feature status
Feature Toggle provides 3 different Views Access checkers:
* **Feature (Unrestricted)**: Only checks the feature status
* **Permission + Feature**: Checks the feature status + a given permission
* **Permission + Role**: Checks the feature status + a given role

### Define Access to Blocks based on feature status
Feature Toggle provides a condition plugin that can be used for Block visibility or
other condition based plugins.

Its name is **Feature Toggle** and allows to define a list of features. If any of them is enabled,
the condition plugin will be evaluated to TRUE.

### Access to features in Twig
Is possible to create conditional logic based on features in twig:
```
{% if feature_toggle_status('my_feature) %}
  // Do something.
{% endif %}
```

## Maintainers

- Pablo López - [plopesc](https://www.drupal.org/u/plopesc)
- Nikhil Dubbaka - [TechNikh](https://www.drupal.org/u/technikh)
