<?php

declare(strict_types=1);

namespace Drupal\feature_toggle\Access;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\feature_toggle\FeatureStatusInterface;
use Symfony\Component\Routing\Route;

/**
 * Provides a checker based on feature toggle statuses.
 *
 * Usage example:
 * @code
 * foo.example:
 *   path: '/example/{parameter}'
 *   defaults:
 *     _title: 'Example'
 *     _controller: '\Drupal\feature_toggle\Controller\FeatureToggleController'
 *   requirements:
 *     _feature_toggle: 'feature.0'
 * @endcode
 */
final class FeatureToggleAccessCheck implements AccessInterface {

  /**
   * Constructs a new FeatureToggleAccessCheck object.
   *
   * @param \Drupal\feature_toggle\FeatureStatusInterface $featureStatus
   *   The feature status.
   */
  public function __construct(
    protected readonly FeatureStatusInterface $featureStatus,
  ) {
  }

  /**
   * Access callback.
   *
   * The route's '_feature_toggle' requirement might follow the pattern
   * 'feature.value'. The value part is optional and it's default value is 1.
   *
   * @param \Symfony\Component\Routing\Route $route
   *   The route to check against.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(Route $route): AccessResultInterface {
    $requirement = $route->getRequirement('_feature_toggle');
    $parts = explode('.', $requirement);
    $feature = $parts[0] ?? '';
    $value = $parts[1] ?? 1;
    $feature_status = $this->featureStatus->getStatus($feature);
    return AccessResult::allowedIf($feature_status == $value)
      ->addCacheTags(['feature_toggle:' . $feature]);
  }

}
