<?php

namespace Drupal\feature_toggle;

use Drupal\Core\Cache\Cache;

/**
 * Service class for FeatureManager logic.
 */
class FeatureManager implements FeatureManagerInterface {

  use FeatureUtilsTrait;

  /**
   * {@inheritdoc}
   */
  public function featureExists(string $name): bool {
    $features = $this->loadFeatures();
    return isset($features[$name]);
  }

  /**
   * {@inheritdoc}
   */
  public function getFeature($name): ?Feature {
    $features = $this->loadFeatures();
    return $features[$name] ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getFeatures(): array {
    return $this->loadFeatures();
  }

  /**
   * {@inheritdoc}
   */
  public function addFeature(FeatureInterface $feature): void {
    $features = $this->loadFeatures();
    if (isset($features[$feature->name()])) {
      throw new \InvalidArgumentException("Feature '{$feature->name()}' already exists.");
    }

    $features[$feature->name()] = $feature;
    $this->saveFeatures($features);
    Cache::invalidateTags([
      'feature_toggle_list',
      'feature_toggle:' . $feature->name(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function updateFeature(FeatureInterface $feature): void {
    $features = $this->loadFeatures();
    if (!isset($features[$feature->name()])) {
      throw new \InvalidArgumentException("Feature '{$feature->name()}' does not exists.");
    }

    $features[$feature->name()] = $feature;
    $this->saveFeatures($features);
    Cache::invalidateTags([
      'feature_toggle_list',
      'feature_toggle:' . $feature->name(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function deleteFeature(string $name): void {
    $this->deleteConfigFeature($name);
    $this->deleteStatusFlag($name);
    Cache::invalidateTags([
      'feature_toggle_list',
      'feature_toggle:' . $name,
    ]);
  }

  /**
   * Deletes the feature form the config object.
   *
   * @param string $name
   *   The feature name.
   */
  protected function deleteConfigFeature(string $name): void {
    $features = $this->loadFeatures();
    unset($features[$name]);
    $this->saveFeatures($features);
  }

  /**
   * Delete the feature from the state array.
   *
   * @param string $name
   *   The feature name.
   */
  protected function deleteStatusFlag(string $name): void {
    $flags = $this->getStatusFlags();
    unset($flags[$name]);
    $this->saveStatusFlags($flags);
  }

}
