<?php

namespace Drupal\feature_toggle;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\KeyValueStore\KeyValueFactoryInterface;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Provides convenience methods for feature toggle services.
 */
trait FeatureUtilsTrait {

  /**
   * The list of features.
   *
   * @var array|null
   */
  protected ?array $features;

  /**
   * Constructs a new FeatureUtilsTrait object.
   */
  public function __construct(
    #[Autowire(service: 'keyvalue')]
    protected KeyValueFactoryInterface $keyValueFactory,
    protected ConfigFactoryInterface $config,
  ) {
  }

  /**
   * Returns the list of features.
   *
   * @return array
   *   The list of features in the system.
   */
  protected function loadFeatures(): array {
    if (!isset($this->features)) {
      $this->features = $this->doLoadFeatures();
    }
    return $this->features;
  }

  /**
   * Loads the list of features from the backend.
   *
   * @return array
   *   The list of features.
   */
  protected function doLoadFeatures(): array {
    return array_map(
      fn (array $feature) => Feature::fromArray($feature),
      $this->config->get('feature_toggle.features')->get('features') ?? []
    );
  }

  /**
   * Saves the features array.
   *
   * @param array $features
   *   The features array.
   */
  protected function saveFeatures(array $features): void {
    $features = array_map(fn (FeatureInterface $feature) => $feature->toArray(), $features);
    $this->config->getEditable('feature_toggle.features')->setData([
      'features' => $features,
    ])->save();
    $this->features = NULL;
  }

  /**
   * Returns the system status flags.
   *
   * @return array
   *   The status flags array.
   */
  public function getStatusFlags(): array {
    return $this->keyValueFactory->get('feature_toggle')->get('flags', []);
  }

  /**
   * Saves the status flags array.
   *
   * @param array $flags
   *   The status flags array.
   */
  protected function saveStatusFlags(array $flags): void {
    $this->keyValueFactory->get('feature_toggle')->set('flags', $flags);
  }

}
