<?php

namespace Drupal\feature_toggle\Form;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\DependencyInjection\AutowireTrait;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\feature_toggle\Feature;
use Drupal\feature_toggle\FeatureInterface;
use Drupal\feature_toggle\FeatureManagerInterface;

/**
 * Form class to add a new feature.
 */
class FeatureForm extends FormBase {

  use AutowireTrait;

  /**
   * The feature being updated, if available.
   *
   * @var \Drupal\feature_toggle\Feature|\Drupal\feature_toggle\FeatureInterface|null
   */
  protected ?FeatureInterface $feature = NULL;

  /**
   * Constructs a new FeatureForm object.
   *
   * @param \Drupal\feature_toggle\FeatureManagerInterface $featureManager
   *   The feature manager.
   */
  public function __construct(
    protected FeatureManagerInterface $featureManager,
  ) {
    if ($this->getRouteMatch()->getParameter('feature_name')) {
      $this->feature = $this->featureManager->getFeature($this->getRouteMatch()->getParameter('feature_name'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'feature_toggle_' . ($this->feature?->name() ? 'edit' : 'add');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    if (isset($this->feature)) {
      $form['#title'] = $this->t('Edit feature %label', ['%label' => $this->feature->label()]);
    }

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Feature Name'),
      '#default_value' => $this->feature?->label(),
      '#required' => TRUE,
    ];
    $form['name'] = [
      '#type' => 'machine_name',
      '#title' => $this->t('Machine name'),
      '#default_value' => $this->feature?->name(),
      '#maxlength' => 64,
      '#description' => $this->t('A unique name for the feature. It must only contain lowercase letters, numbers and hyphens.'),
      '#machine_name' => [
        'exists' => [$this, 'featureNameExists'],
        'source' => ['label'],
      ],
      '#disabled' => !empty($this->feature),
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#default_value' => $this->feature?->description(),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $feature = new Feature(
      $form_state->getValue('name'),
      $form_state->getValue('label'),
      $form_state->getValue('description'),
    );

    $form_id = $this->getFormId();
    $method = match ($this->getFormId()) {
      'feature_toggle_add' => 'addFeature',
      'feature_toggle_edit' => 'updateFeature',
      default => throw new \InvalidArgumentException($this->t('Invalid form ID: @id', ['@id' => $form_id()])),
    };

    $this->featureManager->$method($feature);

    $this->messenger()->addStatus($this->t('Feature <strong>@label</strong> saved successfully.', ['@label' => $form_state->getValue('label')]));
    $form_state->setRedirect('feature_toggle.feature_toggle_form');
  }

  /**
   * Returns whether a feature name already exists.
   *
   * @param string $value
   *   The name of the feature.
   *
   * @return bool
   *   Returns TRUE if the feature already exists, FALSE otherwise.
   */
  public function featureNameExists($value) {
    return $this->featureManager->featureExists($value);
  }

  /**
   * Custom form access checker based on permissions and existing feature name.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user account.
   * @param string $feature_name
   *   The feature name to delete.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result object.
   */
  public function access(AccountInterface $account, $feature_name) {
    return AccessResult::allowedIf($account->hasPermission('administer feature_toggle') && $this->featureManager->featureExists($feature_name));
  }

}
