<?php

namespace Drupal\feature_toggle\Form;

use Drupal\Core\DependencyInjection\AutowireTrait;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\feature_toggle\FeatureManagerInterface;
use Drupal\feature_toggle\FeatureStatusInterface;

/**
 * Lists all the features in a table format and allows to modify their status.
 */
class FeatureToggleForm extends FormBase {

  use AutowireTrait;

  /**
   * Constructs a new FeatureToggleForm object.
   */
  public function __construct(
    protected FeatureManagerInterface $featureManager,
    protected FeatureStatusInterface $featureStatus,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'feature_toggle_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['filters'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['table-filter', 'js-show'],
      ],
    ];

    $form['filters']['text'] = [
      '#type' => 'search',
      '#title' => $this->t('Filter features'),
      '#title_display' => 'invisible',
      '#size' => 30,
      '#placeholder' => $this->t('Filter by name or description'),
      '#description' => $this->t('Enter a part of the feature name'),
      '#attributes' => [
        'class' => ['table-filter-text'],
        'data-table' => '#feature-toggle-form',
        'autocomplete' => 'off',
      ],
    ];

    $form['features'] = [];
    $form['features_status'] = ['#tree' => TRUE];

    $options = [];

    foreach ($this->featureManager->getFeatures() as $feature) {
      if ($this->currentUser()->hasPermission('administer feature_toggle')) {
        $operations = [
          'delete' => [
            '#type' => 'operations',
            '#links' => [
              'edit' => [
                'title' => $this->t('Edit'),
                'url' => Url::fromRoute('feature_toggle.feature_edit_form', ['feature_name' => $feature->name()]),
              ],
              'delete' => [
                'title' => $this->t('Delete'),
                'url' => Url::fromRoute('feature_toggle.feature_delete_form', ['feature_name' => $feature->name()]),
              ],
            ],
          ],
        ];
      }
      else {
        $operations = [
          '#markup' => $this->t('N/A'),
        ];
      }
      $options[$feature->name()] = [
        'machine_name' => $feature->name(),
        'feature' => [
          'data' => [
            '#type' => 'details',
            '#open' => FALSE,
            '#title' => $feature->label(),
            'description' => [
              '#markup' => empty($feature->description()) ? $this->t('No description available.') : $feature->description(),
            ],
          ],
          'class' => ['table-filter-text-source'],
        ],
        'operations' => [
          'data' => $operations,
        ],
      ];
    }

    $form['table'] = [
      '#type' => 'tableselect',
      '#header' => [
        'machine_name' => $this->t('Machine name'),
        'feature' => $this->t('Feature'),
        'operations' => $this->t('Operations'),
      ],
      '#options' => $options,
      '#empty' => $this->t('No features are available to toggle.'),
      '#default_value' => $this->featureStatus->getStatusFlags(),
    ];

    $form['#attached']['library'][] = 'core/drupal.tableresponsive';
    $form['#attached']['library'][] = 'system/drupal.system.modules';
    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    foreach ($this->featureManager->getFeatures() as $feature) {
      $status = (bool) $form_state->getValue(['table', $feature->name()]);
      if ($status !== $this->featureStatus->getStatus($feature->name())) {
        $this->featureStatus->setStatus($feature, $status);
      }
    }

    $this->messenger()->addStatus(t('Feature status saved successfully'));
  }

}
