<?php

declare(strict_types=1);

namespace Drupal\Tests\feature_toggle\Kernel;

use Drupal\feature_toggle\Feature;
use Drupal\feature_toggle\FeatureInterface;
use Drupal\feature_toggle\Plugin\views\access\PermissionFeature;
use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\user\UserInterface;
use Drupal\views\Plugin\views\display\DisplayPluginBase;
use Drupal\views\Tests\ViewTestData;
use Drupal\views\Views;

/**
 * Tests views permission + feature access plugin.
 *
 * @group feature_toggle
 *
 * @see \Drupal\feature_toggle\Plugin\views\access\PermissionFeature
 */
class ViewsAccessPermissionFeatureTest extends KernelTestBase {

  use UserCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'feature_toggle',
    'feature_toggle_test_views',
    'node',
    'system',
    'user',
    'views',
    'views_test_data',
  ];

  /**
   * {@inheritdoc}
   */
  public static $testViews = ['test_access_perm'];

  /**
   * The feature to test.
   *
   * @var \Drupal\feature_toggle\FeatureInterface|\Drupal\feature_toggle\Feature
   */
  protected FeatureInterface $feature;

  /**
   * A user with no special permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $webUser;

  /**
   * A user with 'views_test_data test permission' permission.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $normalUser;

  /**
   * {@inheritdoc}
   */
  protected function setup(): void {
    parent::setUp();

    $this->installEntitySchema('user');
    $this->installEntitySchema('node');

    // Create first UID1 so, the other users are not super-admin.
    $this->createUser([], NULL, FALSE, ['uid' => 1]);
    $this->webUser = $this->createUser();
    $this->normalUser = $this->createUser(['views_test_data test permission']);

    // Create new feature and init it to FALSE.
    $this->feature = new Feature('test_feature', 'Test Feature');
    \Drupal::service('feature_toggle.feature_manager')
      ->addFeature($this->feature);
    \Drupal::service('feature_toggle.feature_status')
      ->setStatus($this->feature, FALSE);

    ViewTestData::createTestViews(static::class, ['feature_toggle_test_views']);
  }

  /**
   * Tests perm access plugin.
   */
  public function testAccessPermTier() {
    $view = Views::getView('test_access_perm');
    $view->setDisplay();

    $access_plugin = $view->display_handler->getPlugin('access');
    $this->assertInstanceOf(PermissionFeature::class, $access_plugin);
    $this->assertEquals('Permission + Feature', $access_plugin->pluginTitle());

    // Should be always FALSE because feature is disabled.
    $this->assertFalse($view->display_handler->access($this->webUser));
    $this->assertFalse($view->display_handler->access($this->normalUser));

    // Enable feature and check again.
    \Drupal::service('feature_toggle.feature_status')
      ->setStatus($this->feature, TRUE);

    // Feature is enabled, now permissions can decide.
    $this->assertFalse($view->display_handler->access($this->webUser));
    $this->assertTrue($view->display_handler->access($this->normalUser));
  }

  /**
   * Tests access on render caching.
   */
  public function testRenderCachingTierFail() {
    $view = Views::getView('test_access_perm');
    $display = &$view->storage->getDisplay('default');
    $display['display_options']['cache'] = [
      'type' => 'tag',
    ];

    /** @var \Drupal\Core\Render\Renderer $renderer */
    $renderer = $this->container->get('renderer');
    $account_switcher = $this->container->get('account_switcher');

    // First access as user without access.
    $build = DisplayPluginBase::buildBasicRenderable('test_access_perm', 'default');
    $account_switcher->switchTo($this->webUser);
    $this->assertEmpty($renderer->renderInIsolation($build));

    // Then with access.
    $build = DisplayPluginBase::buildBasicRenderable('test_access_perm', 'default');
    $account_switcher->switchTo($this->normalUser);
    $this->assertEmpty($renderer->renderInIsolation($build));
  }

  /**
   * Tests access on render caching.
   */
  public function testRenderCachingTierSuccess() {
    // Enable feature and check again.
    \Drupal::service('feature_toggle.feature_status')
      ->setStatus($this->feature, TRUE);

    $view = Views::getView('test_access_perm');
    $display = &$view->storage->getDisplay('default');
    $display['display_options']['cache'] = [
      'type' => 'tag',
    ];

    /** @var \Drupal\Core\Render\Renderer $renderer */
    $renderer = $this->container->get('renderer');
    $account_switcher = $this->container->get('account_switcher');

    // First access as user without access.
    $build = DisplayPluginBase::buildBasicRenderable('test_access_perm', 'default');
    $account_switcher->switchTo($this->webUser);
    $this->assertEmpty($renderer->renderInIsolation($build));

    // Then with access.
    $build = DisplayPluginBase::buildBasicRenderable('test_access_perm', 'default');
    $account_switcher->switchTo($this->normalUser);
    $this->assertNotEmpty($renderer->renderInIsolation($build));
  }

}
